/***
 * Excerpted from "Groovy Recipes",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/sdgrvr for more book information.
***/
// ************* Simple
println ""
def xml = new groovy.xml.StreamingMarkupBuilder().bind{
	person(id:99){
		firstname("Scott")
		"last-name"("Davis")
	}	
}

println xml
// <person id='99'><firstname>Scott</firstname><last-name>Davis</last-name></person>
println ""


// ******************* Deferred
def builder = new groovy.xml.StreamingMarkupBuilder()
/*
def person = {
	person(id:99){
		firstname("Scott")
		lastname("Davis")
	}	
}
System.out << builder.bind(person)

*/// <person id='99'><firstname>Scott</firstname><lastname>Davis</lastname></person>

println ""



// ************************ Write to file
def writer = new FileWriter("person.xml")
//writer << builder.bind(person)


// ************************ Encoding
println ""
println ""

def b = new groovy.xml.StreamingMarkupBuilder()
b.encoding = "UTF-8"
System.out << b.bind{
	mkp.xmlDeclaration()
}

println ""
println ""


// ************************ Namespaces
def b2 = new groovy.xml.StreamingMarkupBuilder().bind{
	mkp.declareNamespace('':'http://myDefaultNamespace')
	mkp.declareNamespace('location':'http://someOtherNamespace')
	person(id:99){
		firstname("Scott")
		lastname("Davis")
		location.address("123 Main St")
	}
}
println ""
println ""
println b2
println ""
println ""

// <person id='99' xmlns='http://myDefaultNamespace' xmlns:location='http://someOtherNamespace'><firstname>Scott</firstname><lastname>Davis</lastname><location:address>123 Main St</location:address></person>



// ******************** comments
def comment = "<!-- address is omitted -->"
def b3 = new groovy.xml.StreamingMarkupBuilder().bind{
	person(id:99){
		firstname("Scott")
		lastname("Davis")
		mkp.yield(comment)
		mkp.yieldUnescaped(comment)
		mkp.yieldUnescaped("<![CDATA[" + comment + "]]>")
		
		out << comment
		unescaped << comment
	}
}
println ""
println ""
println b3
println ""
println ""

//<person id='99'><firstname>Scott</firstname><lastname>Davis</lastname>&lt;!-- address is omitted --&gt;<!-- address is omitted --><![CDATA[<!-- address is omitted -->]]>&lt;!-- address is omitted --&gt;<!-- address is omitted --></person>


// ******************** MKP
def comment = "<!-- address is new to this release -->"
def b4 = new groovy.xml.StreamingMarkupBuilder()
b4.encoding = "UTF-8"
def p = {
	mkp.xmlDeclaration()
	mkp.pi("xml-stylesheet": "type='text/xsl' href='myfile.xslt'" )
	mkp.declareNamespace('':'http://myDefaultNamespace')
	mkp.declareNamespace('location':'http://someOtherNamespace')
	person(id:99){
		firstname("Scott")
		lastname("Davis")
		mkp.yieldUnescaped(comment)
		mkp.yieldUnescaped("<![CDATA[" + comment + "]]>")		
		location.address("123 Main")
	}
}
System.out << blah.bind(p)
println ""
println ""


def h = {
  head{
    title("Search Results")
    link(rel:"stylesheet", type:"text/css", href:"http://main.css")
    script(type:"text/javascript", src:"http://main.js")		
  }
}

def b = {
  body{
    h1("Search Results")
    div(id:"results", class:"simple"){
      table(border:1){
        tr{
          th("Name")
          th("Address")					
        }
        tr{
          td{ 
            a(href:"http://abc.org?id=99","Scott Davis") 
          }
          td("123 Main St")
        }
      }
    }
  }
}

def html = new groovy.xml.StreamingMarkupBuilder().bind{
  unescaped << '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">'
  html{
    out << h
    out << b		
  }
}

System.out << html
println ""

def htmlWriter = new FileWriter("test.html")
htmlWriter << html
